# coding: utf-8

# (C) Copyright IBM Corp. 2024.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# IBM OpenAPI SDK Code Generator Version: 3.87.0-91c7c775-20240320-213027

"""
The Enterprise Management API enables you to create and manage an enterprise, account
groups, and accounts within the enterprise.

API Version: 1.0
"""

from datetime import datetime
from typing import Dict, List, Optional
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse, get_query_param
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import convert_model, datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################


class EnterpriseManagementV1(BaseService):
    """The Enterprise Management V1 service."""

    DEFAULT_SERVICE_URL = 'https://enterprise.cloud.ibm.com/v1'
    DEFAULT_SERVICE_NAME = 'enterprise_management'

    @classmethod
    def new_instance(
        cls,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> 'EnterpriseManagementV1':
        """
        Return a new client for the Enterprise Management service using the
               specified parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(authenticator)
        service.configure_service(service_name)
        return service

    def __init__(
        self,
        authenticator: Authenticator = None,
    ) -> None:
        """
        Construct a new client for the Enterprise Management service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/main/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self, service_url=self.DEFAULT_SERVICE_URL, authenticator=authenticator)

    #########################
    # Enterprise Operations
    #########################

    def create_enterprise(
        self,
        source_account_id: str,
        name: str,
        primary_contact_iam_id: str,
        *,
        domain: Optional[str] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Create an enterprise.

        Create a new enterprise, which you can use to centrally manage multiple accounts.
        To create an enterprise, you must have an active Subscription account.
        <br/><br/>The API creates an enterprise entity, which is the root of the
        enterprise hierarchy. It also creates a new enterprise account that is used to
        manage the enterprise. All subscriptions, support entitlements, credits, and
        discounts from the source subscription account are migrated to the enterprise
        account, and the source account becomes a child account in the hierarchy. The user
        that you assign as the enterprise primary contact is also assigned as the owner of
        the enterprise account.

        :param str source_account_id: The ID of the account that is used to create
               the enterprise.
        :param str name: The name of the enterprise. This field must have 3 - 60
               characters.
        :param str primary_contact_iam_id: The IAM ID of the enterprise primary
               contact, such as `IBMid-0123ABC`. The IAM ID must already exist.
        :param str domain: (optional) A domain or subdomain for the enterprise,
               such as `example.com` or `my.example.com`.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CreateEnterpriseResponse` object
        """

        if source_account_id is None:
            raise ValueError('source_account_id must be provided')
        if name is None:
            raise ValueError('name must be provided')
        if primary_contact_iam_id is None:
            raise ValueError('primary_contact_iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='create_enterprise',
        )
        headers.update(sdk_headers)

        data = {
            'source_account_id': source_account_id,
            'name': name,
            'primary_contact_iam_id': primary_contact_iam_id,
            'domain': domain,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/enterprises'
        request = self.prepare_request(
            method='POST',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def list_enterprises(
        self,
        *,
        enterprise_account_id: Optional[str] = None,
        account_group_id: Optional[str] = None,
        account_id: Optional[str] = None,
        next_docid: Optional[str] = None,
        limit: Optional[int] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        List enterprises.

        Retrieve all enterprises for a given ID by passing the IDs on query parameters. If
        no ID is passed, the enterprises for which the calling identity is the primary
        contact are returned. You can use pagination parameters to filter the results.
        <br/><br/>This method ensures that only the enterprises that the user has access
        to are returned. Access can be controlled either through a policy on a specific
        enterprise, or account-level platform services access roles, such as
        Administrator, Editor, Operator, or Viewer. When you call the method with the
        `enterprise_account_id` or `account_id` query parameter, the account ID in the
        token is compared with that in the query parameter. If these account IDs match,
        authentication isn't performed and the enterprise information is returned. If the
        account IDs don't match, authentication is performed and only then is the
        enterprise information returned in the response.

        :param str enterprise_account_id: (optional) Get enterprises for a given
               enterprise account ID.
        :param str account_group_id: (optional) Get enterprises for a given account
               group ID.
        :param str account_id: (optional) Get enterprises for a given account ID.
        :param str next_docid: (optional) The first item to be returned in the page
               of results. This value can be obtained from the next_url property from the
               previous call of the operation. If not specified, then the first page of
               results is returned.
        :param int limit: (optional) Return results up to this limit. Valid values
               are between `0` and `100`.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListEnterprisesResponse` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='list_enterprises',
        )
        headers.update(sdk_headers)

        params = {
            'enterprise_account_id': enterprise_account_id,
            'account_group_id': account_group_id,
            'account_id': account_id,
            'next_docid': next_docid,
            'limit': limit,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/enterprises'
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    def get_enterprise(
        self,
        enterprise_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get enterprise by ID.

        Retrieve an enterprise by the `enterprise_id` parameter. All data related to the
        enterprise is returned only if the caller has access to retrieve the enterprise.

        :param str enterprise_id: The ID of the enterprise to retrieve.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `Enterprise` object
        """

        if not enterprise_id:
            raise ValueError('enterprise_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_enterprise',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['enterprise_id']
        path_param_values = self.encode_path_vars(enterprise_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/enterprises/{enterprise_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response

    def update_enterprise(
        self,
        enterprise_id: str,
        *,
        name: Optional[str] = None,
        domain: Optional[str] = None,
        primary_contact_iam_id: Optional[str] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Update an enterprise.

        Update the name, domain, or IAM ID of the primary contact for an existing
        enterprise. The new primary contact must already be a user in the enterprise
        account.

        :param str enterprise_id: The ID of the enterprise to retrieve.
        :param str name: (optional) The new name of the enterprise. This field must
               have 3 - 60 characters.
        :param str domain: (optional) The new domain of the enterprise. This field
               has a limit of 60 characters.
        :param str primary_contact_iam_id: (optional) The IAM ID of the user to be
               the new primary contact for the enterprise.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not enterprise_id:
            raise ValueError('enterprise_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='update_enterprise',
        )
        headers.update(sdk_headers)

        data = {
            'name': name,
            'domain': domain,
            'primary_contact_iam_id': primary_contact_iam_id,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['enterprise_id']
        path_param_values = self.encode_path_vars(enterprise_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/enterprises/{enterprise_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='PATCH',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    #########################
    # Account Operations
    #########################

    def import_account_to_enterprise(
        self,
        enterprise_id: str,
        account_id: str,
        *,
        parent: Optional[str] = None,
        billing_unit_id: Optional[str] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Import an account into an enterprise.

        Import an existing stand-alone account into an enterprise. The existing account
        can be any type: trial (`TRIAL`), Lite (`STANDARD`), Pay-As-You-Go (`PAYG`), or
        Subscription (`SUBSCRIPTION`). In the case of a `SUBSCRIPTION` account, the
        credits, promotional offers, and discounts are migrated to the billing unit of the
        enterprise. For a billable account (`PAYG` or `SUBSCRIPTION`), the country and
        currency code of the existing account and the billing unit of the enterprise must
        match. The API returns a `202` response and performs asynchronous operations to
        import the account into the enterprise. <br/></br>For more information about
        impacts to the account, see [Adding accounts to an
        enterprise](https://{DomainName}/docs/account?topic=account-enterprise-add).

        :param str enterprise_id: The ID of the enterprise to import the
               stand-alone account into.
        :param str account_id: The ID of the existing stand-alone account to be
               imported.
        :param str parent: (optional) The CRN of the expected parent of the
               imported account. The parent is the enterprise or account group that the
               account is added to.
        :param str billing_unit_id: (optional) The ID of the [billing
               unit](/apidocs/enterprise-apis/billing-unit) to use for billing this
               account in the enterprise.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not enterprise_id:
            raise ValueError('enterprise_id must be provided')
        if not account_id:
            raise ValueError('account_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='import_account_to_enterprise',
        )
        headers.update(sdk_headers)

        data = {
            'parent': parent,
            'billing_unit_id': billing_unit_id,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['enterprise_id', 'account_id']
        path_param_values = self.encode_path_vars(enterprise_id, account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/enterprises/{enterprise_id}/import/accounts/{account_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='PUT',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def create_account(
        self,
        parent: str,
        name: str,
        owner_iam_id: str,
        *,
        traits: Optional['CreateAccountRequestTraits'] = None,
        options: Optional['CreateAccountRequestOptions'] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Create a new account in an enterprise.

        Create a new account as a part of an existing enterprise. The API creates an
        account entity under the parent that is specified in the payload of the request.
        The request also takes in the name and the owner of this new account. The owner
        must have a valid IBMid that's registered with IBM Cloud, but they don't need to
        be a user in the enterprise account.

        :param str parent: The CRN of the parent under which the account will be
               created. The parent can be an existing account group or the enterprise
               itself.
        :param str name: The name of the account. This field must have 3 - 60
               characters.
        :param str owner_iam_id: The IAM ID of the account owner, such as
               `IBMid-0123ABC`. The IAM ID must already exist.
        :param CreateAccountRequestTraits traits: (optional) The traits object can
               be used to set properties on child accounts of an enterprise. You can pass
               a field to opt-out of the default multi-factor authentication setting or
               enable enterprise-managed IAM when creating a child account in the
               enterprise. This is an optional field.
        :param CreateAccountRequestOptions options: (optional) The options object
               can be used to set properties on child accounts of an enterprise. You can
               pass a field to to create IAM service id with IAM api keyg when creating a
               child account in the enterprise. This is an optional field.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CreateAccountResponse` object
        """

        if parent is None:
            raise ValueError('parent must be provided')
        if name is None:
            raise ValueError('name must be provided')
        if owner_iam_id is None:
            raise ValueError('owner_iam_id must be provided')
        if traits is not None:
            traits = convert_model(traits)
        if options is not None:
            options = convert_model(options)
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='create_account',
        )
        headers.update(sdk_headers)

        data = {
            'parent': parent,
            'name': name,
            'owner_iam_id': owner_iam_id,
            'traits': traits,
            'options': options,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/accounts'
        request = self.prepare_request(
            method='POST',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def list_accounts(
        self,
        *,
        enterprise_id: Optional[str] = None,
        account_group_id: Optional[str] = None,
        next_docid: Optional[str] = None,
        parent: Optional[str] = None,
        limit: Optional[int] = None,
        include_deleted: Optional[bool] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        List accounts.

        Retrieve all accounts based on the values that are passed in the query parameters.
        If no query parameter is passed, all of the accounts in the enterprise for which
        the calling identity has access are returned. <br/><br/>You can use pagination
        parameters to filter the results. The `limit` field can be used to limit the
        number of results that are displayed for this method.<br/><br/>This method ensures
        that only the accounts that the user has access to are returned. Access can be
        controlled either through a policy on a specific account, or account-level
        platform services access roles, such as Administrator, Editor, Operator, or
        Viewer. When you call the method with the `enterprise_id`, `account_group_id` or
        `parent` query parameter, all of the accounts that are immediate children of this
        entity are returned. Authentication is performed on all the accounts before they
        are returned to the user to ensure that only those accounts are returned to which
        the calling identity has access to.

        :param str enterprise_id: (optional) Get accounts that are either immediate
               children or are a part of the hierarchy for a given enterprise ID.
        :param str account_group_id: (optional) Get accounts that are either
               immediate children or are a part of the hierarchy for a given account group
               ID.
        :param str next_docid: (optional) The first item to be returned in the page
               of results. This value can be obtained from the next_url property from the
               previous call of the operation. If not specified, then the first page of
               results is returned.
        :param str parent: (optional) Get accounts that are either immediate
               children or are a part of the hierarchy for a given parent CRN.
        :param int limit: (optional) Return results up to this limit. Valid values
               are between `0` and `100`.
        :param bool include_deleted: (optional) Include the deleted accounts from
               an enterprise when used in conjunction with enterprise_id.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListAccountsResponse` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='list_accounts',
        )
        headers.update(sdk_headers)

        params = {
            'enterprise_id': enterprise_id,
            'account_group_id': account_group_id,
            'next_docid': next_docid,
            'parent': parent,
            'limit': limit,
            'include_deleted': include_deleted,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/accounts'
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    def get_account(
        self,
        account_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get account by ID.

        Retrieve an account by the `account_id` parameter. All data related to the account
        is returned only if the caller has access to retrieve the account.

        :param str account_id: The ID of the target account.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `Account` object
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_account',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/accounts/{account_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response

    def update_account(
        self,
        account_id: str,
        parent: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Move an account within the enterprise.

        Move an account to a different parent within the same enterprise.

        :param str account_id: The ID of the target account.
        :param str parent: The CRN of the new parent within the enterprise.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        if parent is None:
            raise ValueError('parent must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='update_account',
        )
        headers.update(sdk_headers)

        data = {
            'parent': parent,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/accounts/{account_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='PATCH',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def delete_account(
        self,
        account_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Remove an account from its enterprise.

        Remove an account from the enterprise its currently in. After an account is
        removed, it will be canceled and cannot be reactivated.

        :param str account_id: The ID of the target account.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_id:
            raise ValueError('account_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='delete_account',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_id']
        path_param_values = self.encode_path_vars(account_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/accounts/{account_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='DELETE',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response

    #########################
    # Account Group Operations
    #########################

    def create_account_group(
        self,
        parent: str,
        name: str,
        primary_contact_iam_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Create an account group.

        Create a new account group, which can be used to group together multiple accounts.
        To create an account group, you must have an existing enterprise. The API creates
        an account group entity under the parent that is specified in the payload of the
        request. The request also takes in the name and the primary contact of this new
        account group.

        :param str parent: The CRN of the parent under which the account group will
               be created. The parent can be an existing account group or the enterprise
               itself.
        :param str name: The name of the account group. This field must have 3 - 60
               characters.
        :param str primary_contact_iam_id: The IAM ID of the primary contact for
               this account group, such as `IBMid-0123ABC`. The IAM ID must already exist.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CreateAccountGroupResponse` object
        """

        if parent is None:
            raise ValueError('parent must be provided')
        if name is None:
            raise ValueError('name must be provided')
        if primary_contact_iam_id is None:
            raise ValueError('primary_contact_iam_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='create_account_group',
        )
        headers.update(sdk_headers)

        data = {
            'parent': parent,
            'name': name,
            'primary_contact_iam_id': primary_contact_iam_id,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/account-groups'
        request = self.prepare_request(
            method='POST',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def list_account_groups(
        self,
        *,
        enterprise_id: Optional[str] = None,
        parent_account_group_id: Optional[str] = None,
        next_docid: Optional[str] = None,
        parent: Optional[str] = None,
        limit: Optional[int] = None,
        include_deleted: Optional[bool] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        List account groups.

        Retrieve all account groups based on the values that are passed in the query
        parameters. If no query parameter is passed, all of the account groups in the
        enterprise for which the calling identity has access are returned. <br/><br/>You
        can use pagination parameters to filter the results. The `limit` field can be used
        to limit the number of results that are displayed for this method.<br/><br/>This
        method ensures that only the account groups that the user has access to are
        returned. Access can be controlled either through a policy on a specific account
        group, or account-level platform services access roles, such as Administrator,
        Editor, Operator, or Viewer. When you call the method with the `enterprise_id`,
        `parent_account_group_id` or `parent` query parameter, all of the account groups
        that are immediate children of this entity are returned. Authentication is
        performed on all account groups before they are returned to the user to ensure
        that only those account groups are returned to which the calling identity has
        access.

        :param str enterprise_id: (optional) Get account groups that are either
               immediate children or are a part of the hierarchy for a given enterprise
               ID.
        :param str parent_account_group_id: (optional) Get account groups that are
               either immediate children or are a part of the hierarchy for a given
               account group ID.
        :param str next_docid: (optional) The first item to be returned in the page
               of results. This value can be obtained from the next_url property from the
               previous call of the operation. If not specified, then the first page of
               results is returned.
        :param str parent: (optional) Get account groups that are either immediate
               children or are a part of the hierarchy for a given parent CRN.
        :param int limit: (optional) Return results up to this limit. Valid values
               are between `0` and `100`.
        :param bool include_deleted: (optional) Include the deleted account groups
               from an enterprise when used in conjunction with other query parameters.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListAccountGroupsResponse` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='list_account_groups',
        )
        headers.update(sdk_headers)

        params = {
            'enterprise_id': enterprise_id,
            'parent_account_group_id': parent_account_group_id,
            'next_docid': next_docid,
            'parent': parent,
            'limit': limit,
            'include_deleted': include_deleted,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/account-groups'
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    def get_account_group(
        self,
        account_group_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get account group by ID.

        Retrieve an account by the `account_group_id` parameter. All data related to the
        account group is returned only if the caller has access to retrieve the account
        group.

        :param str account_group_id: The ID of the account group to retrieve.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `AccountGroup` object
        """

        if not account_group_id:
            raise ValueError('account_group_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_account_group',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        path_param_keys = ['account_group_id']
        path_param_values = self.encode_path_vars(account_group_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/account-groups/{account_group_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response

    def update_account_group(
        self,
        account_group_id: str,
        *,
        name: Optional[str] = None,
        primary_contact_iam_id: Optional[str] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Update an account group.

        Update the name or IAM ID of the primary contact for an existing account group.
        The new primary contact must already be a user in the enterprise account.

        :param str account_group_id: The ID of the account group to retrieve.
        :param str name: (optional) The new name of the account group. This field
               must have 3 - 60 characters.
        :param str primary_contact_iam_id: (optional) The IAM ID of the user to be
               the new primary contact for the account group.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_group_id:
            raise ValueError('account_group_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='update_account_group',
        )
        headers.update(sdk_headers)

        data = {
            'name': name,
            'primary_contact_iam_id': primary_contact_iam_id,
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_group_id']
        path_param_values = self.encode_path_vars(account_group_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/account-groups/{account_group_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='PATCH',
            url=url,
            headers=headers,
            data=data,
        )

        response = self.send(request, **kwargs)
        return response

    def delete_account_group(
        self,
        account_group_id: str,
        **kwargs,
    ) -> DetailedResponse:
        """
        Delete an account group from the enterprise.

        Delete an existing account group from the enterprise. You can't delete an account
        group that has child account groups, the delete request will fail. This API
        doesn't perform a recursive delete on the child account groups, it only deletes
        the current account group.

        :param str account_group_id: The ID of the account group to retrieve.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if not account_group_id:
            raise ValueError('account_group_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='delete_account_group',
        )
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']

        path_param_keys = ['account_group_id']
        path_param_values = self.encode_path_vars(account_group_id)
        path_param_dict = dict(zip(path_param_keys, path_param_values))
        url = '/account-groups/{account_group_id}'.format(**path_param_dict)
        request = self.prepare_request(
            method='DELETE',
            url=url,
            headers=headers,
        )

        response = self.send(request, **kwargs)
        return response


##############################################################################
# Models
##############################################################################


class Account:
    """
    An account resource.

    :param str url: (optional) The URL of the account.
    :param str id: (optional) The account ID.
    :param str crn: (optional) The Cloud Resource Name (CRN) of the account.
    :param str parent: (optional) The CRN of the parent of the account.
    :param str enterprise_account_id: (optional) The enterprise account ID.
    :param str enterprise_id: (optional) The enterprise ID that the account is a
          part of.
    :param str enterprise_path: (optional) The path from the enterprise to this
          particular account.
    :param str name: (optional) The name of the account.
    :param str state: (optional) The state of the account.
    :param str owner_iam_id: (optional) The IAM ID of the owner of the account.
    :param bool paid: (optional) The type of account - whether it is free or paid.
    :param str owner_email: (optional) The email address of the owner of the
          account.
    :param bool is_enterprise_account: (optional) The flag to indicate whether the
          account is an enterprise account or not.
    :param datetime created_at: (optional) The time stamp at which the account was
          created.
    :param str created_by: (optional) The IAM ID of the user or service that created
          the account.
    :param datetime updated_at: (optional) The time stamp at which the account was
          last updated.
    :param str updated_by: (optional) The IAM ID of the user or service that updated
          the account.
    """

    def __init__(
        self,
        *,
        url: Optional[str] = None,
        id: Optional[str] = None,
        crn: Optional[str] = None,
        parent: Optional[str] = None,
        enterprise_account_id: Optional[str] = None,
        enterprise_id: Optional[str] = None,
        enterprise_path: Optional[str] = None,
        name: Optional[str] = None,
        state: Optional[str] = None,
        owner_iam_id: Optional[str] = None,
        paid: Optional[bool] = None,
        owner_email: Optional[str] = None,
        is_enterprise_account: Optional[bool] = None,
        created_at: Optional[datetime] = None,
        created_by: Optional[str] = None,
        updated_at: Optional[datetime] = None,
        updated_by: Optional[str] = None,
    ) -> None:
        """
        Initialize a Account object.

        :param str url: (optional) The URL of the account.
        :param str id: (optional) The account ID.
        :param str crn: (optional) The Cloud Resource Name (CRN) of the account.
        :param str parent: (optional) The CRN of the parent of the account.
        :param str enterprise_account_id: (optional) The enterprise account ID.
        :param str enterprise_id: (optional) The enterprise ID that the account is
               a part of.
        :param str enterprise_path: (optional) The path from the enterprise to this
               particular account.
        :param str name: (optional) The name of the account.
        :param str state: (optional) The state of the account.
        :param str owner_iam_id: (optional) The IAM ID of the owner of the account.
        :param bool paid: (optional) The type of account - whether it is free or
               paid.
        :param str owner_email: (optional) The email address of the owner of the
               account.
        :param bool is_enterprise_account: (optional) The flag to indicate whether
               the account is an enterprise account or not.
        :param datetime created_at: (optional) The time stamp at which the account
               was created.
        :param str created_by: (optional) The IAM ID of the user or service that
               created the account.
        :param datetime updated_at: (optional) The time stamp at which the account
               was last updated.
        :param str updated_by: (optional) The IAM ID of the user or service that
               updated the account.
        """
        self.url = url
        self.id = id
        self.crn = crn
        self.parent = parent
        self.enterprise_account_id = enterprise_account_id
        self.enterprise_id = enterprise_id
        self.enterprise_path = enterprise_path
        self.name = name
        self.state = state
        self.owner_iam_id = owner_iam_id
        self.paid = paid
        self.owner_email = owner_email
        self.is_enterprise_account = is_enterprise_account
        self.created_at = created_at
        self.created_by = created_by
        self.updated_at = updated_at
        self.updated_by = updated_by

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Account':
        """Initialize a Account object from a json dictionary."""
        args = {}
        if (url := _dict.get('url')) is not None:
            args['url'] = url
        if (id := _dict.get('id')) is not None:
            args['id'] = id
        if (crn := _dict.get('crn')) is not None:
            args['crn'] = crn
        if (parent := _dict.get('parent')) is not None:
            args['parent'] = parent
        if (enterprise_account_id := _dict.get('enterprise_account_id')) is not None:
            args['enterprise_account_id'] = enterprise_account_id
        if (enterprise_id := _dict.get('enterprise_id')) is not None:
            args['enterprise_id'] = enterprise_id
        if (enterprise_path := _dict.get('enterprise_path')) is not None:
            args['enterprise_path'] = enterprise_path
        if (name := _dict.get('name')) is not None:
            args['name'] = name
        if (state := _dict.get('state')) is not None:
            args['state'] = state
        if (owner_iam_id := _dict.get('owner_iam_id')) is not None:
            args['owner_iam_id'] = owner_iam_id
        if (paid := _dict.get('paid')) is not None:
            args['paid'] = paid
        if (owner_email := _dict.get('owner_email')) is not None:
            args['owner_email'] = owner_email
        if (is_enterprise_account := _dict.get('is_enterprise_account')) is not None:
            args['is_enterprise_account'] = is_enterprise_account
        if (created_at := _dict.get('created_at')) is not None:
            args['created_at'] = string_to_datetime(created_at)
        if (created_by := _dict.get('created_by')) is not None:
            args['created_by'] = created_by
        if (updated_at := _dict.get('updated_at')) is not None:
            args['updated_at'] = string_to_datetime(updated_at)
        if (updated_by := _dict.get('updated_by')) is not None:
            args['updated_by'] = updated_by
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Account object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'url') and self.url is not None:
            _dict['url'] = self.url
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'crn') and self.crn is not None:
            _dict['crn'] = self.crn
        if hasattr(self, 'parent') and self.parent is not None:
            _dict['parent'] = self.parent
        if hasattr(self, 'enterprise_account_id') and self.enterprise_account_id is not None:
            _dict['enterprise_account_id'] = self.enterprise_account_id
        if hasattr(self, 'enterprise_id') and self.enterprise_id is not None:
            _dict['enterprise_id'] = self.enterprise_id
        if hasattr(self, 'enterprise_path') and self.enterprise_path is not None:
            _dict['enterprise_path'] = self.enterprise_path
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        if hasattr(self, 'owner_iam_id') and self.owner_iam_id is not None:
            _dict['owner_iam_id'] = self.owner_iam_id
        if hasattr(self, 'paid') and self.paid is not None:
            _dict['paid'] = self.paid
        if hasattr(self, 'owner_email') and self.owner_email is not None:
            _dict['owner_email'] = self.owner_email
        if hasattr(self, 'is_enterprise_account') and self.is_enterprise_account is not None:
            _dict['is_enterprise_account'] = self.is_enterprise_account
        if hasattr(self, 'created_at') and self.created_at is not None:
            _dict['created_at'] = datetime_to_string(self.created_at)
        if hasattr(self, 'created_by') and self.created_by is not None:
            _dict['created_by'] = self.created_by
        if hasattr(self, 'updated_at') and self.updated_at is not None:
            _dict['updated_at'] = datetime_to_string(self.updated_at)
        if hasattr(self, 'updated_by') and self.updated_by is not None:
            _dict['updated_by'] = self.updated_by
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Account object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Account') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Account') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class AccountGroup:
    """
    An account group resource.

    :param str url: (optional) The URL of the account group.
    :param str id: (optional) The account group ID.
    :param str crn: (optional) The Cloud Resource Name (CRN) of the account group.
    :param str parent: (optional) The CRN of the parent of the account group.
    :param str enterprise_account_id: (optional) The enterprise account ID.
    :param str enterprise_id: (optional) The enterprise ID that the account group is
          a part of.
    :param str enterprise_path: (optional) The path from the enterprise to this
          particular account group.
    :param str name: (optional) The name of the account group.
    :param str state: (optional) The state of the account group.
    :param str primary_contact_iam_id: (optional) The IAM ID of the primary contact
          of the account group.
    :param str primary_contact_email: (optional) The email address of the primary
          contact of the account group.
    :param datetime created_at: (optional) The time stamp at which the account group
          was created.
    :param str created_by: (optional) The IAM ID of the user or service that created
          the account group.
    :param datetime updated_at: (optional) The time stamp at which the account group
          was last updated.
    :param str updated_by: (optional) The IAM ID of the user or service that updated
          the account group.
    """

    def __init__(
        self,
        *,
        url: Optional[str] = None,
        id: Optional[str] = None,
        crn: Optional[str] = None,
        parent: Optional[str] = None,
        enterprise_account_id: Optional[str] = None,
        enterprise_id: Optional[str] = None,
        enterprise_path: Optional[str] = None,
        name: Optional[str] = None,
        state: Optional[str] = None,
        primary_contact_iam_id: Optional[str] = None,
        primary_contact_email: Optional[str] = None,
        created_at: Optional[datetime] = None,
        created_by: Optional[str] = None,
        updated_at: Optional[datetime] = None,
        updated_by: Optional[str] = None,
    ) -> None:
        """
        Initialize a AccountGroup object.

        :param str url: (optional) The URL of the account group.
        :param str id: (optional) The account group ID.
        :param str crn: (optional) The Cloud Resource Name (CRN) of the account
               group.
        :param str parent: (optional) The CRN of the parent of the account group.
        :param str enterprise_account_id: (optional) The enterprise account ID.
        :param str enterprise_id: (optional) The enterprise ID that the account
               group is a part of.
        :param str enterprise_path: (optional) The path from the enterprise to this
               particular account group.
        :param str name: (optional) The name of the account group.
        :param str state: (optional) The state of the account group.
        :param str primary_contact_iam_id: (optional) The IAM ID of the primary
               contact of the account group.
        :param str primary_contact_email: (optional) The email address of the
               primary contact of the account group.
        :param datetime created_at: (optional) The time stamp at which the account
               group was created.
        :param str created_by: (optional) The IAM ID of the user or service that
               created the account group.
        :param datetime updated_at: (optional) The time stamp at which the account
               group was last updated.
        :param str updated_by: (optional) The IAM ID of the user or service that
               updated the account group.
        """
        self.url = url
        self.id = id
        self.crn = crn
        self.parent = parent
        self.enterprise_account_id = enterprise_account_id
        self.enterprise_id = enterprise_id
        self.enterprise_path = enterprise_path
        self.name = name
        self.state = state
        self.primary_contact_iam_id = primary_contact_iam_id
        self.primary_contact_email = primary_contact_email
        self.created_at = created_at
        self.created_by = created_by
        self.updated_at = updated_at
        self.updated_by = updated_by

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'AccountGroup':
        """Initialize a AccountGroup object from a json dictionary."""
        args = {}
        if (url := _dict.get('url')) is not None:
            args['url'] = url
        if (id := _dict.get('id')) is not None:
            args['id'] = id
        if (crn := _dict.get('crn')) is not None:
            args['crn'] = crn
        if (parent := _dict.get('parent')) is not None:
            args['parent'] = parent
        if (enterprise_account_id := _dict.get('enterprise_account_id')) is not None:
            args['enterprise_account_id'] = enterprise_account_id
        if (enterprise_id := _dict.get('enterprise_id')) is not None:
            args['enterprise_id'] = enterprise_id
        if (enterprise_path := _dict.get('enterprise_path')) is not None:
            args['enterprise_path'] = enterprise_path
        if (name := _dict.get('name')) is not None:
            args['name'] = name
        if (state := _dict.get('state')) is not None:
            args['state'] = state
        if (primary_contact_iam_id := _dict.get('primary_contact_iam_id')) is not None:
            args['primary_contact_iam_id'] = primary_contact_iam_id
        if (primary_contact_email := _dict.get('primary_contact_email')) is not None:
            args['primary_contact_email'] = primary_contact_email
        if (created_at := _dict.get('created_at')) is not None:
            args['created_at'] = string_to_datetime(created_at)
        if (created_by := _dict.get('created_by')) is not None:
            args['created_by'] = created_by
        if (updated_at := _dict.get('updated_at')) is not None:
            args['updated_at'] = string_to_datetime(updated_at)
        if (updated_by := _dict.get('updated_by')) is not None:
            args['updated_by'] = updated_by
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a AccountGroup object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'url') and self.url is not None:
            _dict['url'] = self.url
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'crn') and self.crn is not None:
            _dict['crn'] = self.crn
        if hasattr(self, 'parent') and self.parent is not None:
            _dict['parent'] = self.parent
        if hasattr(self, 'enterprise_account_id') and self.enterprise_account_id is not None:
            _dict['enterprise_account_id'] = self.enterprise_account_id
        if hasattr(self, 'enterprise_id') and self.enterprise_id is not None:
            _dict['enterprise_id'] = self.enterprise_id
        if hasattr(self, 'enterprise_path') and self.enterprise_path is not None:
            _dict['enterprise_path'] = self.enterprise_path
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        if hasattr(self, 'primary_contact_iam_id') and self.primary_contact_iam_id is not None:
            _dict['primary_contact_iam_id'] = self.primary_contact_iam_id
        if hasattr(self, 'primary_contact_email') and self.primary_contact_email is not None:
            _dict['primary_contact_email'] = self.primary_contact_email
        if hasattr(self, 'created_at') and self.created_at is not None:
            _dict['created_at'] = datetime_to_string(self.created_at)
        if hasattr(self, 'created_by') and self.created_by is not None:
            _dict['created_by'] = self.created_by
        if hasattr(self, 'updated_at') and self.updated_at is not None:
            _dict['updated_at'] = datetime_to_string(self.updated_at)
        if hasattr(self, 'updated_by') and self.updated_by is not None:
            _dict['updated_by'] = self.updated_by
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this AccountGroup object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'AccountGroup') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'AccountGroup') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreateAccountGroupResponse:
    """
    A newly-created account group.

    :param str account_group_id: (optional) The ID of the account group entity that
          was created.
    """

    def __init__(
        self,
        *,
        account_group_id: Optional[str] = None,
    ) -> None:
        """
        Initialize a CreateAccountGroupResponse object.

        :param str account_group_id: (optional) The ID of the account group entity
               that was created.
        """
        self.account_group_id = account_group_id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreateAccountGroupResponse':
        """Initialize a CreateAccountGroupResponse object from a json dictionary."""
        args = {}
        if (account_group_id := _dict.get('account_group_id')) is not None:
            args['account_group_id'] = account_group_id
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreateAccountGroupResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'account_group_id') and self.account_group_id is not None:
            _dict['account_group_id'] = self.account_group_id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreateAccountGroupResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreateAccountGroupResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreateAccountGroupResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreateAccountRequestOptions:
    """
    The options object can be used to set properties on child accounts of an enterprise.
    You can pass a field to to create IAM service id with IAM api keyg when creating a
    child account in the enterprise. This is an optional field.

    :param bool create_iam_service_id_with_apikey_and_owner_policies: (optional) By
          default create_iam_service_id_with_apikey_and_owner_policies is turned off for a
          newly created child account. You can enable this property by passing 'true' in
          this boolean field. IAM service id has account owner IAM policies and the API
          key associated with it can generate a token and setup resources in the account.
          This is an optional field.
    """

    def __init__(
        self,
        *,
        create_iam_service_id_with_apikey_and_owner_policies: Optional[bool] = None,
    ) -> None:
        """
        Initialize a CreateAccountRequestOptions object.

        :param bool create_iam_service_id_with_apikey_and_owner_policies:
               (optional) By default create_iam_service_id_with_apikey_and_owner_policies
               is turned off for a newly created child account. You can enable this
               property by passing 'true' in this boolean field. IAM service id has
               account owner IAM policies and the API key associated with it can generate
               a token and setup resources in the account. This is an optional field.
        """
        self.create_iam_service_id_with_apikey_and_owner_policies = create_iam_service_id_with_apikey_and_owner_policies

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreateAccountRequestOptions':
        """Initialize a CreateAccountRequestOptions object from a json dictionary."""
        args = {}
        if (
            create_iam_service_id_with_apikey_and_owner_policies := _dict.get(
                'create_iam_service_id_with_apikey_and_owner_policies'
            )
        ) is not None:
            args['create_iam_service_id_with_apikey_and_owner_policies'] = (
                create_iam_service_id_with_apikey_and_owner_policies
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreateAccountRequestOptions object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if (
            hasattr(self, 'create_iam_service_id_with_apikey_and_owner_policies')
            and self.create_iam_service_id_with_apikey_and_owner_policies is not None
        ):
            _dict['create_iam_service_id_with_apikey_and_owner_policies'] = (
                self.create_iam_service_id_with_apikey_and_owner_policies
            )
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreateAccountRequestOptions object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreateAccountRequestOptions') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreateAccountRequestOptions') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreateAccountRequestTraits:
    """
    The traits object can be used to set properties on child accounts of an enterprise.
    You can pass a field to opt-out of the default multi-factor authentication setting or
    enable enterprise-managed IAM when creating a child account in the enterprise. This is
    an optional field.

    :param str mfa: (optional) By default MFA is set to `NONE_NO_ROPC` on a child
          account, which disables CLI logins with only a password. To opt out, pass the
          traits object with the mfa field set to empty string. This is an optional field.
    :param bool enterprise_iam_managed: (optional) By default enterprise-managed IAM
          is turned off for a newly created child account. You can enable this property by
          passing 'true' in this boolean field. Enabling enterprise-managed IAM allows the
          enterprise account to assign IAM resources, like access groups, trusted
          profiles, and account settings, to the child account. This is an optional field.
    """

    def __init__(
        self,
        *,
        mfa: Optional[str] = None,
        enterprise_iam_managed: Optional[bool] = None,
    ) -> None:
        """
        Initialize a CreateAccountRequestTraits object.

        :param str mfa: (optional) By default MFA is set to `NONE_NO_ROPC` on a
               child account, which disables CLI logins with only a password. To opt out,
               pass the traits object with the mfa field set to empty string. This is an
               optional field.
        :param bool enterprise_iam_managed: (optional) By default
               enterprise-managed IAM is turned off for a newly created child account. You
               can enable this property by passing 'true' in this boolean field. Enabling
               enterprise-managed IAM allows the enterprise account to assign IAM
               resources, like access groups, trusted profiles, and account settings, to
               the child account. This is an optional field.
        """
        self.mfa = mfa
        self.enterprise_iam_managed = enterprise_iam_managed

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreateAccountRequestTraits':
        """Initialize a CreateAccountRequestTraits object from a json dictionary."""
        args = {}
        if (mfa := _dict.get('mfa')) is not None:
            args['mfa'] = mfa
        if (enterprise_iam_managed := _dict.get('enterprise_iam_managed')) is not None:
            args['enterprise_iam_managed'] = enterprise_iam_managed
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreateAccountRequestTraits object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'mfa') and self.mfa is not None:
            _dict['mfa'] = self.mfa
        if hasattr(self, 'enterprise_iam_managed') and self.enterprise_iam_managed is not None:
            _dict['enterprise_iam_managed'] = self.enterprise_iam_managed
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreateAccountRequestTraits object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreateAccountRequestTraits') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreateAccountRequestTraits') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreateAccountResponse:
    """
    A newly-created account.

    :param str account_id: (optional) The ID of the account entity that was created.
    """

    def __init__(
        self,
        *,
        account_id: Optional[str] = None,
    ) -> None:
        """
        Initialize a CreateAccountResponse object.

        :param str account_id: (optional) The ID of the account entity that was
               created.
        """
        self.account_id = account_id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreateAccountResponse':
        """Initialize a CreateAccountResponse object from a json dictionary."""
        args = {}
        if (account_id := _dict.get('account_id')) is not None:
            args['account_id'] = account_id
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreateAccountResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'account_id') and self.account_id is not None:
            _dict['account_id'] = self.account_id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreateAccountResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreateAccountResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreateAccountResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreateEnterpriseResponse:
    """
    The response from calling create enterprise.

    :param str enterprise_id: (optional) The ID of the enterprise entity that was
          created. This entity is the root of the hierarchy.
    :param str enterprise_account_id: (optional) The ID of the enterprise account
          that was created. The enterprise account is used to manage billing and access to
          the enterprise management.
    """

    def __init__(
        self,
        *,
        enterprise_id: Optional[str] = None,
        enterprise_account_id: Optional[str] = None,
    ) -> None:
        """
        Initialize a CreateEnterpriseResponse object.

        :param str enterprise_id: (optional) The ID of the enterprise entity that
               was created. This entity is the root of the hierarchy.
        :param str enterprise_account_id: (optional) The ID of the enterprise
               account that was created. The enterprise account is used to manage billing
               and access to the enterprise management.
        """
        self.enterprise_id = enterprise_id
        self.enterprise_account_id = enterprise_account_id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreateEnterpriseResponse':
        """Initialize a CreateEnterpriseResponse object from a json dictionary."""
        args = {}
        if (enterprise_id := _dict.get('enterprise_id')) is not None:
            args['enterprise_id'] = enterprise_id
        if (enterprise_account_id := _dict.get('enterprise_account_id')) is not None:
            args['enterprise_account_id'] = enterprise_account_id
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreateEnterpriseResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'enterprise_id') and self.enterprise_id is not None:
            _dict['enterprise_id'] = self.enterprise_id
        if hasattr(self, 'enterprise_account_id') and self.enterprise_account_id is not None:
            _dict['enterprise_account_id'] = self.enterprise_account_id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreateEnterpriseResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreateEnterpriseResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreateEnterpriseResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Enterprise:
    """
    An enterprise resource.

    :param str url: (optional) The URL of the enterprise.
    :param str id: (optional) The enterprise ID.
    :param str enterprise_account_id: (optional) The enterprise account ID.
    :param str crn: (optional) The Cloud Resource Name (CRN) of the enterprise.
    :param str name: (optional) The name of the enterprise.
    :param str domain: (optional) The domain of the enterprise.
    :param str state: (optional) The state of the enterprise.
    :param str primary_contact_iam_id: (optional) The IAM ID of the primary contact
          of the enterprise, such as `IBMid-0123ABC`.
    :param str primary_contact_email: (optional) The email of the primary contact of
          the enterprise.
    :param str source_account_id: (optional) The ID of the account that is used to
          create the enterprise.
    :param datetime created_at: (optional) The time stamp at which the enterprise
          was created.
    :param str created_by: (optional) The IAM ID of the user or service that created
          the enterprise.
    :param datetime updated_at: (optional) The time stamp at which the enterprise
          was last updated.
    :param str updated_by: (optional) The IAM ID of the user or service that updated
          the enterprise.
    """

    def __init__(
        self,
        *,
        url: Optional[str] = None,
        id: Optional[str] = None,
        enterprise_account_id: Optional[str] = None,
        crn: Optional[str] = None,
        name: Optional[str] = None,
        domain: Optional[str] = None,
        state: Optional[str] = None,
        primary_contact_iam_id: Optional[str] = None,
        primary_contact_email: Optional[str] = None,
        source_account_id: Optional[str] = None,
        created_at: Optional[datetime] = None,
        created_by: Optional[str] = None,
        updated_at: Optional[datetime] = None,
        updated_by: Optional[str] = None,
    ) -> None:
        """
        Initialize a Enterprise object.

        :param str url: (optional) The URL of the enterprise.
        :param str id: (optional) The enterprise ID.
        :param str enterprise_account_id: (optional) The enterprise account ID.
        :param str crn: (optional) The Cloud Resource Name (CRN) of the enterprise.
        :param str name: (optional) The name of the enterprise.
        :param str domain: (optional) The domain of the enterprise.
        :param str state: (optional) The state of the enterprise.
        :param str primary_contact_iam_id: (optional) The IAM ID of the primary
               contact of the enterprise, such as `IBMid-0123ABC`.
        :param str primary_contact_email: (optional) The email of the primary
               contact of the enterprise.
        :param str source_account_id: (optional) The ID of the account that is used
               to create the enterprise.
        :param datetime created_at: (optional) The time stamp at which the
               enterprise was created.
        :param str created_by: (optional) The IAM ID of the user or service that
               created the enterprise.
        :param datetime updated_at: (optional) The time stamp at which the
               enterprise was last updated.
        :param str updated_by: (optional) The IAM ID of the user or service that
               updated the enterprise.
        """
        self.url = url
        self.id = id
        self.enterprise_account_id = enterprise_account_id
        self.crn = crn
        self.name = name
        self.domain = domain
        self.state = state
        self.primary_contact_iam_id = primary_contact_iam_id
        self.primary_contact_email = primary_contact_email
        self.source_account_id = source_account_id
        self.created_at = created_at
        self.created_by = created_by
        self.updated_at = updated_at
        self.updated_by = updated_by

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Enterprise':
        """Initialize a Enterprise object from a json dictionary."""
        args = {}
        if (url := _dict.get('url')) is not None:
            args['url'] = url
        if (id := _dict.get('id')) is not None:
            args['id'] = id
        if (enterprise_account_id := _dict.get('enterprise_account_id')) is not None:
            args['enterprise_account_id'] = enterprise_account_id
        if (crn := _dict.get('crn')) is not None:
            args['crn'] = crn
        if (name := _dict.get('name')) is not None:
            args['name'] = name
        if (domain := _dict.get('domain')) is not None:
            args['domain'] = domain
        if (state := _dict.get('state')) is not None:
            args['state'] = state
        if (primary_contact_iam_id := _dict.get('primary_contact_iam_id')) is not None:
            args['primary_contact_iam_id'] = primary_contact_iam_id
        if (primary_contact_email := _dict.get('primary_contact_email')) is not None:
            args['primary_contact_email'] = primary_contact_email
        if (source_account_id := _dict.get('source_account_id')) is not None:
            args['source_account_id'] = source_account_id
        if (created_at := _dict.get('created_at')) is not None:
            args['created_at'] = string_to_datetime(created_at)
        if (created_by := _dict.get('created_by')) is not None:
            args['created_by'] = created_by
        if (updated_at := _dict.get('updated_at')) is not None:
            args['updated_at'] = string_to_datetime(updated_at)
        if (updated_by := _dict.get('updated_by')) is not None:
            args['updated_by'] = updated_by
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Enterprise object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'url') and self.url is not None:
            _dict['url'] = self.url
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'enterprise_account_id') and self.enterprise_account_id is not None:
            _dict['enterprise_account_id'] = self.enterprise_account_id
        if hasattr(self, 'crn') and self.crn is not None:
            _dict['crn'] = self.crn
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'domain') and self.domain is not None:
            _dict['domain'] = self.domain
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        if hasattr(self, 'primary_contact_iam_id') and self.primary_contact_iam_id is not None:
            _dict['primary_contact_iam_id'] = self.primary_contact_iam_id
        if hasattr(self, 'primary_contact_email') and self.primary_contact_email is not None:
            _dict['primary_contact_email'] = self.primary_contact_email
        if hasattr(self, 'source_account_id') and self.source_account_id is not None:
            _dict['source_account_id'] = self.source_account_id
        if hasattr(self, 'created_at') and self.created_at is not None:
            _dict['created_at'] = datetime_to_string(self.created_at)
        if hasattr(self, 'created_by') and self.created_by is not None:
            _dict['created_by'] = self.created_by
        if hasattr(self, 'updated_at') and self.updated_at is not None:
            _dict['updated_at'] = datetime_to_string(self.updated_at)
        if hasattr(self, 'updated_by') and self.updated_by is not None:
            _dict['updated_by'] = self.updated_by
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Enterprise object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Enterprise') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Enterprise') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ListAccountGroupsResponse:
    """
    The list_account_groups operation response.

    :param int rows_count: (optional) The number of enterprises returned from
          calling list account groups.
    :param str next_url: (optional) A string that represents the link to the next
          page of results.
    :param List[AccountGroup] resources: (optional) A list of account groups.
    """

    def __init__(
        self,
        *,
        rows_count: Optional[int] = None,
        next_url: Optional[str] = None,
        resources: Optional[List['AccountGroup']] = None,
    ) -> None:
        """
        Initialize a ListAccountGroupsResponse object.

        :param int rows_count: (optional) The number of enterprises returned from
               calling list account groups.
        :param str next_url: (optional) A string that represents the link to the
               next page of results.
        :param List[AccountGroup] resources: (optional) A list of account groups.
        """
        self.rows_count = rows_count
        self.next_url = next_url
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListAccountGroupsResponse':
        """Initialize a ListAccountGroupsResponse object from a json dictionary."""
        args = {}
        if (rows_count := _dict.get('rows_count')) is not None:
            args['rows_count'] = rows_count
        if (next_url := _dict.get('next_url')) is not None:
            args['next_url'] = next_url
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = [AccountGroup.from_dict(v) for v in resources]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListAccountGroupsResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'rows_count') and self.rows_count is not None:
            _dict['rows_count'] = self.rows_count
        if hasattr(self, 'next_url') and self.next_url is not None:
            _dict['next_url'] = self.next_url
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListAccountGroupsResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListAccountGroupsResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListAccountGroupsResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ListAccountsResponse:
    """
    The list_accounts operation response.

    :param int rows_count: (optional) The number of enterprises returned from
          calling list accounts.
    :param str next_url: (optional) A string that represents the link to the next
          page of results.
    :param List[Account] resources: (optional) A list of accounts.
    """

    def __init__(
        self,
        *,
        rows_count: Optional[int] = None,
        next_url: Optional[str] = None,
        resources: Optional[List['Account']] = None,
    ) -> None:
        """
        Initialize a ListAccountsResponse object.

        :param int rows_count: (optional) The number of enterprises returned from
               calling list accounts.
        :param str next_url: (optional) A string that represents the link to the
               next page of results.
        :param List[Account] resources: (optional) A list of accounts.
        """
        self.rows_count = rows_count
        self.next_url = next_url
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListAccountsResponse':
        """Initialize a ListAccountsResponse object from a json dictionary."""
        args = {}
        if (rows_count := _dict.get('rows_count')) is not None:
            args['rows_count'] = rows_count
        if (next_url := _dict.get('next_url')) is not None:
            args['next_url'] = next_url
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = [Account.from_dict(v) for v in resources]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListAccountsResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'rows_count') and self.rows_count is not None:
            _dict['rows_count'] = self.rows_count
        if hasattr(self, 'next_url') and self.next_url is not None:
            _dict['next_url'] = self.next_url
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListAccountsResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListAccountsResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListAccountsResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ListEnterprisesResponse:
    """
    The response from calling list enterprises.

    :param int rows_count: (optional) The number of enterprises returned from
          calling list enterprise.
    :param str next_url: (optional) A string that represents the link to the next
          page of results.
    :param List[Enterprise] resources: (optional) A list of enterprise objects.
    """

    def __init__(
        self,
        *,
        rows_count: Optional[int] = None,
        next_url: Optional[str] = None,
        resources: Optional[List['Enterprise']] = None,
    ) -> None:
        """
        Initialize a ListEnterprisesResponse object.

        :param int rows_count: (optional) The number of enterprises returned from
               calling list enterprise.
        :param str next_url: (optional) A string that represents the link to the
               next page of results.
        :param List[Enterprise] resources: (optional) A list of enterprise objects.
        """
        self.rows_count = rows_count
        self.next_url = next_url
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListEnterprisesResponse':
        """Initialize a ListEnterprisesResponse object from a json dictionary."""
        args = {}
        if (rows_count := _dict.get('rows_count')) is not None:
            args['rows_count'] = rows_count
        if (next_url := _dict.get('next_url')) is not None:
            args['next_url'] = next_url
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = [Enterprise.from_dict(v) for v in resources]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListEnterprisesResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'rows_count') and self.rows_count is not None:
            _dict['rows_count'] = self.rows_count
        if hasattr(self, 'next_url') and self.next_url is not None:
            _dict['next_url'] = self.next_url
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListEnterprisesResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListEnterprisesResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListEnterprisesResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


##############################################################################
# Pagers
##############################################################################


class EnterprisesPager:
    """
    EnterprisesPager can be used to simplify the use of the "list_enterprises" method.
    """

    def __init__(
        self,
        *,
        client: EnterpriseManagementV1,
        enterprise_account_id: str = None,
        account_group_id: str = None,
        account_id: str = None,
        limit: int = None,
    ) -> None:
        """
        Initialize a EnterprisesPager object.
        :param str enterprise_account_id: (optional) Get enterprises for a given
               enterprise account ID.
        :param str account_group_id: (optional) Get enterprises for a given account
               group ID.
        :param str account_id: (optional) Get enterprises for a given account ID.
        :param int limit: (optional) Return results up to this limit. Valid values
               are between `0` and `100`.
        """
        self._has_next = True
        self._client = client
        self._page_context = {'next': None}
        self._enterprise_account_id = enterprise_account_id
        self._account_group_id = account_group_id
        self._account_id = account_id
        self._limit = limit

    def has_next(self) -> bool:
        """
        Returns true if there are potentially more results to be retrieved.
        """
        return self._has_next

    def get_next(self) -> List[dict]:
        """
        Returns the next page of results.
        :return: A List[dict], where each element is a dict that represents an instance of Enterprise.
        :rtype: List[dict]
        """
        if not self.has_next():
            raise StopIteration(message='No more results available')

        result = self._client.list_enterprises(
            enterprise_account_id=self._enterprise_account_id,
            account_group_id=self._account_group_id,
            account_id=self._account_id,
            limit=self._limit,
            next_docid=self._page_context.get('next'),
        ).get_result()

        next = None
        next_page_link = result.get('next_url')
        if next_page_link is not None:
            next = get_query_param(next_page_link, 'next_docid')
        self._page_context['next'] = next
        if next is None:
            self._has_next = False

        return result.get('resources')

    def get_all(self) -> List[dict]:
        """
        Returns all results by invoking get_next() repeatedly
        until all pages of results have been retrieved.
        :return: A List[dict], where each element is a dict that represents an instance of Enterprise.
        :rtype: List[dict]
        """
        results = []
        while self.has_next():
            next_page = self.get_next()
            results.extend(next_page)
        return results


class AccountsPager:
    """
    AccountsPager can be used to simplify the use of the "list_accounts" method.
    """

    def __init__(
        self,
        *,
        client: EnterpriseManagementV1,
        enterprise_id: str = None,
        account_group_id: str = None,
        parent: str = None,
        limit: int = None,
        include_deleted: bool = None,
    ) -> None:
        """
        Initialize a AccountsPager object.
        :param str enterprise_id: (optional) Get accounts that are either immediate
               children or are a part of the hierarchy for a given enterprise ID.
        :param str account_group_id: (optional) Get accounts that are either
               immediate children or are a part of the hierarchy for a given account group
               ID.
        :param str parent: (optional) Get accounts that are either immediate
               children or are a part of the hierarchy for a given parent CRN.
        :param int limit: (optional) Return results up to this limit. Valid values
               are between `0` and `100`.
        :param bool include_deleted: (optional) Include the deleted accounts from
               an enterprise when used in conjunction with enterprise_id.
        """
        self._has_next = True
        self._client = client
        self._page_context = {'next': None}
        self._enterprise_id = enterprise_id
        self._account_group_id = account_group_id
        self._parent = parent
        self._limit = limit
        self._include_deleted = include_deleted

    def has_next(self) -> bool:
        """
        Returns true if there are potentially more results to be retrieved.
        """
        return self._has_next

    def get_next(self) -> List[dict]:
        """
        Returns the next page of results.
        :return: A List[dict], where each element is a dict that represents an instance of Account.
        :rtype: List[dict]
        """
        if not self.has_next():
            raise StopIteration(message='No more results available')

        result = self._client.list_accounts(
            enterprise_id=self._enterprise_id,
            account_group_id=self._account_group_id,
            parent=self._parent,
            limit=self._limit,
            include_deleted=self._include_deleted,
            next_docid=self._page_context.get('next'),
        ).get_result()

        next = None
        next_page_link = result.get('next_url')
        if next_page_link is not None:
            next = get_query_param(next_page_link, 'next_docid')
        self._page_context['next'] = next
        if next is None:
            self._has_next = False

        return result.get('resources')

    def get_all(self) -> List[dict]:
        """
        Returns all results by invoking get_next() repeatedly
        until all pages of results have been retrieved.
        :return: A List[dict], where each element is a dict that represents an instance of Account.
        :rtype: List[dict]
        """
        results = []
        while self.has_next():
            next_page = self.get_next()
            results.extend(next_page)
        return results


class AccountGroupsPager:
    """
    AccountGroupsPager can be used to simplify the use of the "list_account_groups" method.
    """

    def __init__(
        self,
        *,
        client: EnterpriseManagementV1,
        enterprise_id: str = None,
        parent_account_group_id: str = None,
        parent: str = None,
        limit: int = None,
        include_deleted: bool = None,
    ) -> None:
        """
        Initialize a AccountGroupsPager object.
        :param str enterprise_id: (optional) Get account groups that are either
               immediate children or are a part of the hierarchy for a given enterprise
               ID.
        :param str parent_account_group_id: (optional) Get account groups that are
               either immediate children or are a part of the hierarchy for a given
               account group ID.
        :param str parent: (optional) Get account groups that are either immediate
               children or are a part of the hierarchy for a given parent CRN.
        :param int limit: (optional) Return results up to this limit. Valid values
               are between `0` and `100`.
        :param bool include_deleted: (optional) Include the deleted account groups
               from an enterprise when used in conjunction with other query parameters.
        """
        self._has_next = True
        self._client = client
        self._page_context = {'next': None}
        self._enterprise_id = enterprise_id
        self._parent_account_group_id = parent_account_group_id
        self._parent = parent
        self._limit = limit
        self._include_deleted = include_deleted

    def has_next(self) -> bool:
        """
        Returns true if there are potentially more results to be retrieved.
        """
        return self._has_next

    def get_next(self) -> List[dict]:
        """
        Returns the next page of results.
        :return: A List[dict], where each element is a dict that represents an instance of AccountGroup.
        :rtype: List[dict]
        """
        if not self.has_next():
            raise StopIteration(message='No more results available')

        result = self._client.list_account_groups(
            enterprise_id=self._enterprise_id,
            parent_account_group_id=self._parent_account_group_id,
            parent=self._parent,
            limit=self._limit,
            include_deleted=self._include_deleted,
            next_docid=self._page_context.get('next'),
        ).get_result()

        next = None
        next_page_link = result.get('next_url')
        if next_page_link is not None:
            next = get_query_param(next_page_link, 'next_docid')
        self._page_context['next'] = next
        if next is None:
            self._has_next = False

        return result.get('resources')

    def get_all(self) -> List[dict]:
        """
        Returns all results by invoking get_next() repeatedly
        until all pages of results have been retrieved.
        :return: A List[dict], where each element is a dict that represents an instance of AccountGroup.
        :rtype: List[dict]
        """
        results = []
        while self.has_next():
            next_page = self.get_next()
            results.extend(next_page)
        return results



# coding: utf-8

# (C) Copyright IBM Corp. 2024.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# IBM OpenAPI SDK Code Generator Version: 3.85.0-75c38f8f-20240206-210220

"""
Billing units for IBM Cloud partners

API Version: 1.0.0
"""

from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################


class PartnerBillingUnitsV1(BaseService):
    """The Partner Billing Units V1 service."""

    DEFAULT_SERVICE_URL = 'https://partner.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'partner_billing_units'

    @classmethod
    def new_instance(
        cls,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> 'PartnerBillingUnitsV1':
        """
        Return a new client for the Partner Billing Units service using the
               specified parameters and external configuration.
        """
        authenticator = get_authenticator_from_environment(service_name)
        service = cls(authenticator)
        service.configure_service(service_name)
        return service

    def __init__(
        self,
        authenticator: Authenticator = None,
    ) -> None:
        """
        Construct a new client for the Partner Billing Units service.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/main/README.md
               about initializing the authenticator of your choice.
        """
        BaseService.__init__(self, service_url=self.DEFAULT_SERVICE_URL, authenticator=authenticator)

    #########################
    # Billing Options
    #########################

    def get_billing_options(
        self,
        partner_id: str,
        *,
        customer_id: Optional[str] = None,
        reseller_id: Optional[str] = None,
        date: Optional[str] = None,
        limit: Optional[int] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get customers billing options.

        Returns the billing options for the requested customer for a given month.

        :param str partner_id: Enterprise ID of the distributor or reseller for
               which the report is requested.
        :param str customer_id: (optional) Enterprise ID of the customer for which
               the report is requested.
        :param str reseller_id: (optional) Enterprise ID of the reseller for which
               the report is requested.
        :param str date: (optional) The billing month for which the usage report is
               requested. Format is yyyy-mm. Defaults to current month.
        :param int limit: (optional) Number of usage records returned. The default
               value is 30. Maximum value is 200.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `BillingOptionsSummary` object
        """

        if not partner_id:
            raise ValueError('partner_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_billing_options',
        )
        headers.update(sdk_headers)

        params = {
            'partner_id': partner_id,
            'customer_id': customer_id,
            'reseller_id': reseller_id,
            'date': date,
            '_limit': limit,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/v1/billing-options'
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response

    #########################
    # Credit Pools
    #########################

    def get_credit_pools_report(
        self,
        partner_id: str,
        *,
        customer_id: Optional[str] = None,
        reseller_id: Optional[str] = None,
        date: Optional[str] = None,
        limit: Optional[int] = None,
        **kwargs,
    ) -> DetailedResponse:
        """
        Get subscription burn-down report.

        Returns the subscription or commitment burn-down reports for the end customers for
        a given month.

        :param str partner_id: Enterprise ID of the distributor or reseller for
               which the report is requested.
        :param str customer_id: (optional) Enterprise ID of the customer for which
               the report is requested.
        :param str reseller_id: (optional) Enterprise ID of the reseller for which
               the report is requested.
        :param str date: (optional) The billing month for which the usage report is
               requested. Format is yyyy-mm. Defaults to current month.
        :param int limit: (optional) Number of usage records returned. The default
               value is 30. Maximum value is 200.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CreditPoolsReportSummary` object
        """

        if not partner_id:
            raise ValueError('partner_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(
            service_name=self.DEFAULT_SERVICE_NAME,
            service_version='V1',
            operation_id='get_credit_pools_report',
        )
        headers.update(sdk_headers)

        params = {
            'partner_id': partner_id,
            'customer_id': customer_id,
            'reseller_id': reseller_id,
            'date': date,
            '_limit': limit,
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
            del kwargs['headers']
        headers['Accept'] = 'application/json'

        url = '/v1/credit-pools'
        request = self.prepare_request(
            method='GET',
            url=url,
            headers=headers,
            params=params,
        )

        response = self.send(request, **kwargs)
        return response


##############################################################################
# Models
##############################################################################


class BillingOptionsSummaryFirst:
    """
    The link to the first page of the search query.

    :param str href: (optional) A link to a page of query results.
    """

    def __init__(
        self,
        *,
        href: Optional[str] = None,
    ) -> None:
        """
        Initialize a BillingOptionsSummaryFirst object.

        :param str href: (optional) A link to a page of query results.
        """
        self.href = href

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'BillingOptionsSummaryFirst':
        """Initialize a BillingOptionsSummaryFirst object from a json dictionary."""
        args = {}
        if (href := _dict.get('href')) is not None:
            args['href'] = href
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a BillingOptionsSummaryFirst object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'href') and self.href is not None:
            _dict['href'] = self.href
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this BillingOptionsSummaryFirst object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'BillingOptionsSummaryFirst') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'BillingOptionsSummaryFirst') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class BillingOptionsSummaryNext:
    """
    The link to the next page of the search query.

    :param str href: (optional) A link to a page of query results.
    :param str offset: (optional) The value of the `_start` query parameter to fetch
          the next page.
    """

    def __init__(
        self,
        *,
        href: Optional[str] = None,
        offset: Optional[str] = None,
    ) -> None:
        """
        Initialize a BillingOptionsSummaryNext object.

        :param str href: (optional) A link to a page of query results.
        :param str offset: (optional) The value of the `_start` query parameter to
               fetch the next page.
        """
        self.href = href
        self.offset = offset

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'BillingOptionsSummaryNext':
        """Initialize a BillingOptionsSummaryNext object from a json dictionary."""
        args = {}
        if (href := _dict.get('href')) is not None:
            args['href'] = href
        if (offset := _dict.get('offset')) is not None:
            args['offset'] = offset
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a BillingOptionsSummaryNext object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'href') and self.href is not None:
            _dict['href'] = self.href
        if hasattr(self, 'offset') and self.offset is not None:
            _dict['offset'] = self.offset
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this BillingOptionsSummaryNext object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'BillingOptionsSummaryNext') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'BillingOptionsSummaryNext') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class BillingOption:
    """
    Billing options report for the end customers.

    :param str id: (optional) The ID of the billing option.
    :param str billing_unit_id: (optional) The ID of the billing unit that's
          associated with the billing option.
    :param str customer_id: (optional) Account ID of the customer.
    :param str customer_type: (optional) The customer type. The valid values are
          `ENTERPRISE`, `ACCOUNT`, and `ACCOUNT_GROUP`.
    :param str customer_name: (optional) A user-defined name for the customer.
    :param str reseller_id: (optional) ID of the reseller in the heirarchy of the
          requested customer.
    :param str reseller_name: (optional) Name of the reseller in the heirarchy of
          the requested customer.
    :param str month: (optional) The billing month for which the burn-down report is
          requested. Format is yyyy-mm. Defaults to current month.
    :param List[dict] errors: (optional) Errors in the billing.
    :param str type: (optional) The type of billing option. The valid values are
          `SUBSCRIPTION` and `OFFER`.
    :param datetime start_date: (optional) The start date of billing option.
    :param datetime end_date: (optional) The end date of billing option.
    :param str state: (optional) The state of the billing option. The valid values
          include `ACTIVE, `SUSPENDED`, and `CANCELED`.
    :param str category: (optional) The category of the billing option. The valid
          values are `PLATFORM`, `SERVICE`, and `SUPPORT`.
    :param dict payment_instrument: (optional) The payment method for support.
    :param str part_number: (optional) Part number of the offering.
    :param str catalog_id: (optional) ID of the catalog containing this offering.
    :param str order_id: (optional) ID of the order containing this offering.
    :param str po_number: (optional) PO Number of the offering.
    :param str subscription_model: (optional) Subscription model.
    :param int duration_in_months: (optional) The duration of the billing options in
          months.
    :param float monthly_amount: (optional) Amount billed monthly for this offering.
    :param dict billing_system: (optional) The support billing system.
    :param str country_code: (optional) The country code for the billing unit.
    :param str currency_code: (optional) The currency code of the billing unit.
    """

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        billing_unit_id: Optional[str] = None,
        customer_id: Optional[str] = None,
        customer_type: Optional[str] = None,
        customer_name: Optional[str] = None,
        reseller_id: Optional[str] = None,
        reseller_name: Optional[str] = None,
        month: Optional[str] = None,
        errors: Optional[List[dict]] = None,
        type: Optional[str] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
        state: Optional[str] = None,
        category: Optional[str] = None,
        payment_instrument: Optional[dict] = None,
        part_number: Optional[str] = None,
        catalog_id: Optional[str] = None,
        order_id: Optional[str] = None,
        po_number: Optional[str] = None,
        subscription_model: Optional[str] = None,
        duration_in_months: Optional[int] = None,
        monthly_amount: Optional[float] = None,
        billing_system: Optional[dict] = None,
        country_code: Optional[str] = None,
        currency_code: Optional[str] = None,
    ) -> None:
        """
        Initialize a BillingOption object.

        :param str id: (optional) The ID of the billing option.
        :param str billing_unit_id: (optional) The ID of the billing unit that's
               associated with the billing option.
        :param str customer_id: (optional) Account ID of the customer.
        :param str customer_type: (optional) The customer type. The valid values
               are `ENTERPRISE`, `ACCOUNT`, and `ACCOUNT_GROUP`.
        :param str customer_name: (optional) A user-defined name for the customer.
        :param str reseller_id: (optional) ID of the reseller in the heirarchy of
               the requested customer.
        :param str reseller_name: (optional) Name of the reseller in the heirarchy
               of the requested customer.
        :param str month: (optional) The billing month for which the burn-down
               report is requested. Format is yyyy-mm. Defaults to current month.
        :param List[dict] errors: (optional) Errors in the billing.
        :param str type: (optional) The type of billing option. The valid values
               are `SUBSCRIPTION` and `OFFER`.
        :param datetime start_date: (optional) The start date of billing option.
        :param datetime end_date: (optional) The end date of billing option.
        :param str state: (optional) The state of the billing option. The valid
               values include `ACTIVE, `SUSPENDED`, and `CANCELED`.
        :param str category: (optional) The category of the billing option. The
               valid values are `PLATFORM`, `SERVICE`, and `SUPPORT`.
        :param dict payment_instrument: (optional) The payment method for support.
        :param str part_number: (optional) Part number of the offering.
        :param str catalog_id: (optional) ID of the catalog containing this
               offering.
        :param str order_id: (optional) ID of the order containing this offering.
        :param str po_number: (optional) PO Number of the offering.
        :param str subscription_model: (optional) Subscription model.
        :param int duration_in_months: (optional) The duration of the billing
               options in months.
        :param float monthly_amount: (optional) Amount billed monthly for this
               offering.
        :param dict billing_system: (optional) The support billing system.
        :param str country_code: (optional) The country code for the billing unit.
        :param str currency_code: (optional) The currency code of the billing unit.
        """
        self.id = id
        self.billing_unit_id = billing_unit_id
        self.customer_id = customer_id
        self.customer_type = customer_type
        self.customer_name = customer_name
        self.reseller_id = reseller_id
        self.reseller_name = reseller_name
        self.month = month
        self.errors = errors
        self.type = type
        self.start_date = start_date
        self.end_date = end_date
        self.state = state
        self.category = category
        self.payment_instrument = payment_instrument
        self.part_number = part_number
        self.catalog_id = catalog_id
        self.order_id = order_id
        self.po_number = po_number
        self.subscription_model = subscription_model
        self.duration_in_months = duration_in_months
        self.monthly_amount = monthly_amount
        self.billing_system = billing_system
        self.country_code = country_code
        self.currency_code = currency_code

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'BillingOption':
        """Initialize a BillingOption object from a json dictionary."""
        args = {}
        if (id := _dict.get('id')) is not None:
            args['id'] = id
        if (billing_unit_id := _dict.get('billing_unit_id')) is not None:
            args['billing_unit_id'] = billing_unit_id
        if (customer_id := _dict.get('customer_id')) is not None:
            args['customer_id'] = customer_id
        if (customer_type := _dict.get('customer_type')) is not None:
            args['customer_type'] = customer_type
        if (customer_name := _dict.get('customer_name')) is not None:
            args['customer_name'] = customer_name
        if (reseller_id := _dict.get('reseller_id')) is not None:
            args['reseller_id'] = reseller_id
        if (reseller_name := _dict.get('reseller_name')) is not None:
            args['reseller_name'] = reseller_name
        if (month := _dict.get('month')) is not None:
            args['month'] = month
        if (errors := _dict.get('errors')) is not None:
            args['errors'] = errors
        if (type := _dict.get('type')) is not None:
            args['type'] = type
        if (start_date := _dict.get('start_date')) is not None:
            args['start_date'] = string_to_datetime(start_date)
        if (end_date := _dict.get('end_date')) is not None:
            args['end_date'] = string_to_datetime(end_date)
        if (state := _dict.get('state')) is not None:
            args['state'] = state
        if (category := _dict.get('category')) is not None:
            args['category'] = category
        if (payment_instrument := _dict.get('payment_instrument')) is not None:
            args['payment_instrument'] = payment_instrument
        if (part_number := _dict.get('part_number')) is not None:
            args['part_number'] = part_number
        if (catalog_id := _dict.get('catalog_id')) is not None:
            args['catalog_id'] = catalog_id
        if (order_id := _dict.get('order_id')) is not None:
            args['order_id'] = order_id
        if (po_number := _dict.get('po_number')) is not None:
            args['po_number'] = po_number
        if (subscription_model := _dict.get('subscription_model')) is not None:
            args['subscription_model'] = subscription_model
        if (duration_in_months := _dict.get('duration_in_months')) is not None:
            args['duration_in_months'] = duration_in_months
        if (monthly_amount := _dict.get('monthly_amount')) is not None:
            args['monthly_amount'] = monthly_amount
        if (billing_system := _dict.get('billing_system')) is not None:
            args['billing_system'] = billing_system
        if (country_code := _dict.get('country_code')) is not None:
            args['country_code'] = country_code
        if (currency_code := _dict.get('currency_code')) is not None:
            args['currency_code'] = currency_code
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a BillingOption object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'billing_unit_id') and self.billing_unit_id is not None:
            _dict['billing_unit_id'] = self.billing_unit_id
        if hasattr(self, 'customer_id') and self.customer_id is not None:
            _dict['customer_id'] = self.customer_id
        if hasattr(self, 'customer_type') and self.customer_type is not None:
            _dict['customer_type'] = self.customer_type
        if hasattr(self, 'customer_name') and self.customer_name is not None:
            _dict['customer_name'] = self.customer_name
        if hasattr(self, 'reseller_id') and self.reseller_id is not None:
            _dict['reseller_id'] = self.reseller_id
        if hasattr(self, 'reseller_name') and self.reseller_name is not None:
            _dict['reseller_name'] = self.reseller_name
        if hasattr(self, 'month') and self.month is not None:
            _dict['month'] = self.month
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'start_date') and self.start_date is not None:
            _dict['start_date'] = datetime_to_string(self.start_date)
        if hasattr(self, 'end_date') and self.end_date is not None:
            _dict['end_date'] = datetime_to_string(self.end_date)
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        if hasattr(self, 'category') and self.category is not None:
            _dict['category'] = self.category
        if hasattr(self, 'payment_instrument') and self.payment_instrument is not None:
            _dict['payment_instrument'] = self.payment_instrument
        if hasattr(self, 'part_number') and self.part_number is not None:
            _dict['part_number'] = self.part_number
        if hasattr(self, 'catalog_id') and self.catalog_id is not None:
            _dict['catalog_id'] = self.catalog_id
        if hasattr(self, 'order_id') and self.order_id is not None:
            _dict['order_id'] = self.order_id
        if hasattr(self, 'po_number') and self.po_number is not None:
            _dict['po_number'] = self.po_number
        if hasattr(self, 'subscription_model') and self.subscription_model is not None:
            _dict['subscription_model'] = self.subscription_model
        if hasattr(self, 'duration_in_months') and self.duration_in_months is not None:
            _dict['duration_in_months'] = self.duration_in_months
        if hasattr(self, 'monthly_amount') and self.monthly_amount is not None:
            _dict['monthly_amount'] = self.monthly_amount
        if hasattr(self, 'billing_system') and self.billing_system is not None:
            _dict['billing_system'] = self.billing_system
        if hasattr(self, 'country_code') and self.country_code is not None:
            _dict['country_code'] = self.country_code
        if hasattr(self, 'currency_code') and self.currency_code is not None:
            _dict['currency_code'] = self.currency_code
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this BillingOption object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'BillingOption') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'BillingOption') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class CustomerTypeEnum(str, Enum):
        """
        The customer type. The valid values are `ENTERPRISE`, `ACCOUNT`, and
        `ACCOUNT_GROUP`.
        """

        ENTERPRISE = 'ENTERPRISE'
        ACCOUNT = 'ACCOUNT'
        ACCOUNT_GROUP = 'ACCOUNT_GROUP'

    class TypeEnum(str, Enum):
        """
        The type of billing option. The valid values are `SUBSCRIPTION` and `OFFER`.
        """

        SUBSCRIPTION = 'SUBSCRIPTION'
        OFFER = 'OFFER'

    class StateEnum(str, Enum):
        """
        The state of the billing option. The valid values include `ACTIVE, `SUSPENDED`,
        and `CANCELED`.
        """

        ACTIVE = 'ACTIVE'
        SUSPENDED = 'SUSPENDED'
        CANCELED = 'CANCELED'

    class CategoryEnum(str, Enum):
        """
        The category of the billing option. The valid values are `PLATFORM`, `SERVICE`,
        and `SUPPORT`.
        """

        PLATFORM = 'PLATFORM'
        SERVICE = 'SERVICE'
        SUPPORT = 'SUPPORT'


class BillingOptionsSummary:
    """
    The billing options report for the customer.

    :param int limit: (optional) The max number of reports in the response.
    :param BillingOptionsSummaryFirst first: (optional) The link to the first page
          of the search query.
    :param BillingOptionsSummaryNext next: (optional) The link to the next page of
          the search query.
    :param List[BillingOption] resources: (optional) Aggregated usage report of all
          requested partners.
    """

    def __init__(
        self,
        *,
        limit: Optional[int] = None,
        first: Optional['BillingOptionsSummaryFirst'] = None,
        next: Optional['BillingOptionsSummaryNext'] = None,
        resources: Optional[List['BillingOption']] = None,
    ) -> None:
        """
        Initialize a BillingOptionsSummary object.

        :param int limit: (optional) The max number of reports in the response.
        :param BillingOptionsSummaryFirst first: (optional) The link to the first
               page of the search query.
        :param BillingOptionsSummaryNext next: (optional) The link to the next page
               of the search query.
        :param List[BillingOption] resources: (optional) Aggregated usage report of
               all requested partners.
        """
        self.limit = limit
        self.first = first
        self.next = next
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'BillingOptionsSummary':
        """Initialize a BillingOptionsSummary object from a json dictionary."""
        args = {}
        if (limit := _dict.get('limit')) is not None:
            args['limit'] = limit
        if (first := _dict.get('first')) is not None:
            args['first'] = BillingOptionsSummaryFirst.from_dict(first)
        if (next := _dict.get('next')) is not None:
            args['next'] = BillingOptionsSummaryNext.from_dict(next)
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = [BillingOption.from_dict(v) for v in resources]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a BillingOptionsSummary object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'limit') and self.limit is not None:
            _dict['limit'] = self.limit
        if hasattr(self, 'first') and self.first is not None:
            if isinstance(self.first, dict):
                _dict['first'] = self.first
            else:
                _dict['first'] = self.first.to_dict()
        if hasattr(self, 'next') and self.next is not None:
            if isinstance(self.next, dict):
                _dict['next'] = self.next
            else:
                _dict['next'] = self.next.to_dict()
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this BillingOptionsSummary object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'BillingOptionsSummary') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'BillingOptionsSummary') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreditPoolsReportSummaryFirst:
    """
    The link to the first page of the search query.

    :param str href: (optional) A link to a page of query results.
    """

    def __init__(
        self,
        *,
        href: Optional[str] = None,
    ) -> None:
        """
        Initialize a CreditPoolsReportSummaryFirst object.

        :param str href: (optional) A link to a page of query results.
        """
        self.href = href

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreditPoolsReportSummaryFirst':
        """Initialize a CreditPoolsReportSummaryFirst object from a json dictionary."""
        args = {}
        if (href := _dict.get('href')) is not None:
            args['href'] = href
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreditPoolsReportSummaryFirst object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'href') and self.href is not None:
            _dict['href'] = self.href
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreditPoolsReportSummaryFirst object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreditPoolsReportSummaryFirst') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreditPoolsReportSummaryFirst') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreditPoolsReportSummaryNext:
    """
    The link to the next page of the search query.

    :param str href: (optional) A link to a page of query results.
    :param str offset: (optional) The value of the `_start` query parameter to fetch
          the next page.
    """

    def __init__(
        self,
        *,
        href: Optional[str] = None,
        offset: Optional[str] = None,
    ) -> None:
        """
        Initialize a CreditPoolsReportSummaryNext object.

        :param str href: (optional) A link to a page of query results.
        :param str offset: (optional) The value of the `_start` query parameter to
               fetch the next page.
        """
        self.href = href
        self.offset = offset

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreditPoolsReportSummaryNext':
        """Initialize a CreditPoolsReportSummaryNext object from a json dictionary."""
        args = {}
        if (href := _dict.get('href')) is not None:
            args['href'] = href
        if (offset := _dict.get('offset')) is not None:
            args['offset'] = offset
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreditPoolsReportSummaryNext object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'href') and self.href is not None:
            _dict['href'] = self.href
        if hasattr(self, 'offset') and self.offset is not None:
            _dict['offset'] = self.offset
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreditPoolsReportSummaryNext object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreditPoolsReportSummaryNext') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreditPoolsReportSummaryNext') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class CreditPoolsReport:
    """
    Aggregated subscription burn-down report for the end customers.

    :param str type: (optional) The category of the billing option. The valid values
          are `PLATFORM`, `SERVICE` and `SUPPORT`.
    :param str billing_unit_id: (optional) The ID of the billing unit that's
          associated with the billing option.
    :param str customer_id: (optional) Account ID of the customer.
    :param str customer_type: (optional) The customer type. The valid values are
          `ENTERPRISE`, `ACCOUNT`, and `ACCOUNT_GROUP`.
    :param str customer_name: (optional) A user-defined name for the customer.
    :param str reseller_id: (optional) ID of the reseller in the heirarchy of the
          requested customer.
    :param str reseller_name: (optional) Name of the reseller in the heirarchy of
          the requested customer.
    :param str month: (optional) The billing month for which the burn-down report is
          requested. Format is yyyy-mm. Defaults to current month.
    :param str currency_code: (optional) The currency code of the billing unit.
    :param List[TermCredits] term_credits: (optional) A list of active subscription
          terms available within a credit.
    :param Overage overage: (optional) Overage that was generated on the credit
          pool.
    """

    def __init__(
        self,
        *,
        type: Optional[str] = None,
        billing_unit_id: Optional[str] = None,
        customer_id: Optional[str] = None,
        customer_type: Optional[str] = None,
        customer_name: Optional[str] = None,
        reseller_id: Optional[str] = None,
        reseller_name: Optional[str] = None,
        month: Optional[str] = None,
        currency_code: Optional[str] = None,
        term_credits: Optional[List['TermCredits']] = None,
        overage: Optional['Overage'] = None,
    ) -> None:
        """
        Initialize a CreditPoolsReport object.

        :param str type: (optional) The category of the billing option. The valid
               values are `PLATFORM`, `SERVICE` and `SUPPORT`.
        :param str billing_unit_id: (optional) The ID of the billing unit that's
               associated with the billing option.
        :param str customer_id: (optional) Account ID of the customer.
        :param str customer_type: (optional) The customer type. The valid values
               are `ENTERPRISE`, `ACCOUNT`, and `ACCOUNT_GROUP`.
        :param str customer_name: (optional) A user-defined name for the customer.
        :param str reseller_id: (optional) ID of the reseller in the heirarchy of
               the requested customer.
        :param str reseller_name: (optional) Name of the reseller in the heirarchy
               of the requested customer.
        :param str month: (optional) The billing month for which the burn-down
               report is requested. Format is yyyy-mm. Defaults to current month.
        :param str currency_code: (optional) The currency code of the billing unit.
        :param List[TermCredits] term_credits: (optional) A list of active
               subscription terms available within a credit.
        :param Overage overage: (optional) Overage that was generated on the credit
               pool.
        """
        self.type = type
        self.billing_unit_id = billing_unit_id
        self.customer_id = customer_id
        self.customer_type = customer_type
        self.customer_name = customer_name
        self.reseller_id = reseller_id
        self.reseller_name = reseller_name
        self.month = month
        self.currency_code = currency_code
        self.term_credits = term_credits
        self.overage = overage

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreditPoolsReport':
        """Initialize a CreditPoolsReport object from a json dictionary."""
        args = {}
        if (type := _dict.get('type')) is not None:
            args['type'] = type
        if (billing_unit_id := _dict.get('billing_unit_id')) is not None:
            args['billing_unit_id'] = billing_unit_id
        if (customer_id := _dict.get('customer_id')) is not None:
            args['customer_id'] = customer_id
        if (customer_type := _dict.get('customer_type')) is not None:
            args['customer_type'] = customer_type
        if (customer_name := _dict.get('customer_name')) is not None:
            args['customer_name'] = customer_name
        if (reseller_id := _dict.get('reseller_id')) is not None:
            args['reseller_id'] = reseller_id
        if (reseller_name := _dict.get('reseller_name')) is not None:
            args['reseller_name'] = reseller_name
        if (month := _dict.get('month')) is not None:
            args['month'] = month
        if (currency_code := _dict.get('currency_code')) is not None:
            args['currency_code'] = currency_code
        if (term_credits := _dict.get('term_credits')) is not None:
            args['term_credits'] = [TermCredits.from_dict(v) for v in term_credits]
        if (overage := _dict.get('overage')) is not None:
            args['overage'] = Overage.from_dict(overage)
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreditPoolsReport object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'type') and self.type is not None:
            _dict['type'] = self.type
        if hasattr(self, 'billing_unit_id') and self.billing_unit_id is not None:
            _dict['billing_unit_id'] = self.billing_unit_id
        if hasattr(self, 'customer_id') and self.customer_id is not None:
            _dict['customer_id'] = self.customer_id
        if hasattr(self, 'customer_type') and self.customer_type is not None:
            _dict['customer_type'] = self.customer_type
        if hasattr(self, 'customer_name') and self.customer_name is not None:
            _dict['customer_name'] = self.customer_name
        if hasattr(self, 'reseller_id') and self.reseller_id is not None:
            _dict['reseller_id'] = self.reseller_id
        if hasattr(self, 'reseller_name') and self.reseller_name is not None:
            _dict['reseller_name'] = self.reseller_name
        if hasattr(self, 'month') and self.month is not None:
            _dict['month'] = self.month
        if hasattr(self, 'currency_code') and self.currency_code is not None:
            _dict['currency_code'] = self.currency_code
        if hasattr(self, 'term_credits') and self.term_credits is not None:
            term_credits_list = []
            for v in self.term_credits:
                if isinstance(v, dict):
                    term_credits_list.append(v)
                else:
                    term_credits_list.append(v.to_dict())
            _dict['term_credits'] = term_credits_list
        if hasattr(self, 'overage') and self.overage is not None:
            if isinstance(self.overage, dict):
                _dict['overage'] = self.overage
            else:
                _dict['overage'] = self.overage.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreditPoolsReport object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreditPoolsReport') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreditPoolsReport') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class TypeEnum(str, Enum):
        """
        The category of the billing option. The valid values are `PLATFORM`, `SERVICE` and
        `SUPPORT`.
        """

        PLATFORM = 'PLATFORM'
        SERVICE = 'SERVICE'
        SUPPORT = 'SUPPORT'

    class CustomerTypeEnum(str, Enum):
        """
        The customer type. The valid values are `ENTERPRISE`, `ACCOUNT`, and
        `ACCOUNT_GROUP`.
        """

        ENTERPRISE = 'ENTERPRISE'
        ACCOUNT = 'ACCOUNT'
        ACCOUNT_GROUP = 'ACCOUNT_GROUP'


class CreditPoolsReportSummary:
    """
    The aggregated credit pools report.

    :param int limit: (optional) The max number of reports in the response.
    :param CreditPoolsReportSummaryFirst first: (optional) The link to the first
          page of the search query.
    :param CreditPoolsReportSummaryNext next: (optional) The link to the next page
          of the search query.
    :param List[CreditPoolsReport] resources: (optional) Aggregated usage report of
          all requested partners.
    """

    def __init__(
        self,
        *,
        limit: Optional[int] = None,
        first: Optional['CreditPoolsReportSummaryFirst'] = None,
        next: Optional['CreditPoolsReportSummaryNext'] = None,
        resources: Optional[List['CreditPoolsReport']] = None,
    ) -> None:
        """
        Initialize a CreditPoolsReportSummary object.

        :param int limit: (optional) The max number of reports in the response.
        :param CreditPoolsReportSummaryFirst first: (optional) The link to the
               first page of the search query.
        :param CreditPoolsReportSummaryNext next: (optional) The link to the next
               page of the search query.
        :param List[CreditPoolsReport] resources: (optional) Aggregated usage
               report of all requested partners.
        """
        self.limit = limit
        self.first = first
        self.next = next
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CreditPoolsReportSummary':
        """Initialize a CreditPoolsReportSummary object from a json dictionary."""
        args = {}
        if (limit := _dict.get('limit')) is not None:
            args['limit'] = limit
        if (first := _dict.get('first')) is not None:
            args['first'] = CreditPoolsReportSummaryFirst.from_dict(first)
        if (next := _dict.get('next')) is not None:
            args['next'] = CreditPoolsReportSummaryNext.from_dict(next)
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = [CreditPoolsReport.from_dict(v) for v in resources]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CreditPoolsReportSummary object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'limit') and self.limit is not None:
            _dict['limit'] = self.limit
        if hasattr(self, 'first') and self.first is not None:
            if isinstance(self.first, dict):
                _dict['first'] = self.first
            else:
                _dict['first'] = self.first.to_dict()
        if hasattr(self, 'next') and self.next is not None:
            if isinstance(self.next, dict):
                _dict['next'] = self.next
            else:
                _dict['next'] = self.next.to_dict()
        if hasattr(self, 'resources') and self.resources is not None:
            resources_list = []
            for v in self.resources:
                if isinstance(v, dict):
                    resources_list.append(v)
                else:
                    resources_list.append(v.to_dict())
            _dict['resources'] = resources_list
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CreditPoolsReportSummary object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CreditPoolsReportSummary') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CreditPoolsReportSummary') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Overage:
    """
    Overage that was generated on the credit pool.

    :param float cost: (optional) The number of credits used as overage.
    :param List[dict] resources: (optional) A list of resources that generated
          overage.
    """

    def __init__(
        self,
        *,
        cost: Optional[float] = None,
        resources: Optional[List[dict]] = None,
    ) -> None:
        """
        Initialize a Overage object.

        :param float cost: (optional) The number of credits used as overage.
        :param List[dict] resources: (optional) A list of resources that generated
               overage.
        """
        self.cost = cost
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Overage':
        """Initialize a Overage object from a json dictionary."""
        args = {}
        if (cost := _dict.get('cost')) is not None:
            args['cost'] = cost
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = resources
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Overage object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'cost') and self.cost is not None:
            _dict['cost'] = self.cost
        if hasattr(self, 'resources') and self.resources is not None:
            _dict['resources'] = self.resources
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Overage object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'Overage') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Overage') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class TermCredits:
    """
    The subscription term that is active in the requested month.

    :param str billing_option_id: (optional) The ID of the billing option from which
          the subscription term is derived.
    :param str billing_option_model: (optional) Billing option model.
    :param str category: (optional) The category of the billing option. The valid
          values are `PLATFORM`, `SERVICE`, and `SUPPORT`.
    :param datetime start_date: (optional) The start date of the term in ISO format.
    :param datetime end_date: (optional) The end date of the term in ISO format.
    :param float total_credits: (optional) The total credit available in this term.
    :param float starting_balance: (optional) The balance of available credit at the
          start of the current month.
    :param float used_credits: (optional) The amount of credit used during the
          current month.
    :param float current_balance: (optional) The balance of remaining credit in the
          subscription term.
    :param List[dict] resources: (optional) A list of resources that used credit
          during the month.
    """

    def __init__(
        self,
        *,
        billing_option_id: Optional[str] = None,
        billing_option_model: Optional[str] = None,
        category: Optional[str] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
        total_credits: Optional[float] = None,
        starting_balance: Optional[float] = None,
        used_credits: Optional[float] = None,
        current_balance: Optional[float] = None,
        resources: Optional[List[dict]] = None,
    ) -> None:
        """
        Initialize a TermCredits object.

        :param str billing_option_id: (optional) The ID of the billing option from
               which the subscription term is derived.
        :param str billing_option_model: (optional) Billing option model.
        :param str category: (optional) The category of the billing option. The
               valid values are `PLATFORM`, `SERVICE`, and `SUPPORT`.
        :param datetime start_date: (optional) The start date of the term in ISO
               format.
        :param datetime end_date: (optional) The end date of the term in ISO
               format.
        :param float total_credits: (optional) The total credit available in this
               term.
        :param float starting_balance: (optional) The balance of available credit
               at the start of the current month.
        :param float used_credits: (optional) The amount of credit used during the
               current month.
        :param float current_balance: (optional) The balance of remaining credit in
               the subscription term.
        :param List[dict] resources: (optional) A list of resources that used
               credit during the month.
        """
        self.billing_option_id = billing_option_id
        self.billing_option_model = billing_option_model
        self.category = category
        self.start_date = start_date
        self.end_date = end_date
        self.total_credits = total_credits
        self.starting_balance = starting_balance
        self.used_credits = used_credits
        self.current_balance = current_balance
        self.resources = resources

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'TermCredits':
        """Initialize a TermCredits object from a json dictionary."""
        args = {}
        if (billing_option_id := _dict.get('billing_option_id')) is not None:
            args['billing_option_id'] = billing_option_id
        if (billing_option_model := _dict.get('billing_option_model')) is not None:
            args['billing_option_model'] = billing_option_model
        if (category := _dict.get('category')) is not None:
            args['category'] = category
        if (start_date := _dict.get('start_date')) is not None:
            args['start_date'] = string_to_datetime(start_date)
        if (end_date := _dict.get('end_date')) is not None:
            args['end_date'] = string_to_datetime(end_date)
        if (total_credits := _dict.get('total_credits')) is not None:
            args['total_credits'] = total_credits
        if (starting_balance := _dict.get('starting_balance')) is not None:
            args['starting_balance'] = starting_balance
        if (used_credits := _dict.get('used_credits')) is not None:
            args['used_credits'] = used_credits
        if (current_balance := _dict.get('current_balance')) is not None:
            args['current_balance'] = current_balance
        if (resources := _dict.get('resources')) is not None:
            args['resources'] = resources
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a TermCredits object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'billing_option_id') and self.billing_option_id is not None:
            _dict['billing_option_id'] = self.billing_option_id
        if hasattr(self, 'billing_option_model') and self.billing_option_model is not None:
            _dict['billing_option_model'] = self.billing_option_model
        if hasattr(self, 'category') and self.category is not None:
            _dict['category'] = self.category
        if hasattr(self, 'start_date') and self.start_date is not None:
            _dict['start_date'] = datetime_to_string(self.start_date)
        if hasattr(self, 'end_date') and self.end_date is not None:
            _dict['end_date'] = datetime_to_string(self.end_date)
        if hasattr(self, 'total_credits') and self.total_credits is not None:
            _dict['total_credits'] = self.total_credits
        if hasattr(self, 'starting_balance') and self.starting_balance is not None:
            _dict['starting_balance'] = self.starting_balance
        if hasattr(self, 'used_credits') and self.used_credits is not None:
            _dict['used_credits'] = self.used_credits
        if hasattr(self, 'current_balance') and self.current_balance is not None:
            _dict['current_balance'] = self.current_balance
        if hasattr(self, 'resources') and self.resources is not None:
            _dict['resources'] = self.resources
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this TermCredits object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'TermCredits') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'TermCredits') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class CategoryEnum(str, Enum):
        """
        The category of the billing option. The valid values are `PLATFORM`, `SERVICE`,
        and `SUPPORT`.
        """

        PLATFORM = 'PLATFORM'
        SERVICE = 'SERVICE'
        SUPPORT = 'SUPPORT'
